unit Slider;

interface

{
File:        Slider
Project:     Fixed, Modified and Enhanced Bonanza Component
Version:     1.00
Author:      Lon Voogd
Date:        27. Juli 1996}

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Forms,
  Controls, Dialogs, StdCtrls, ExtCtrls, DsgnIntf;

type
  TAlign       = (alNone, alTop, alBottom, alClient);
  TSlider      = (esNoChange, esSlider);
  TPanelState  = (None, Single, Raised, Lowered, Group, InvGroup);
  TOnChange    = procedure (Sender: TObject; var lPosition: Longint; tWhichSlider: TSlider) of object;
  TOnChanging  = procedure (Sender: TObject; var lSlidePos: Longint; tWhichSlider: TSlider) of object;

type
  TLSlider = class(TGraphicControl)
    private
      fAlign:       TAlign;                           {Contains internal alignment}
      lMaxValue:    Longint;                          {max position of the slider}
      lMinValue:    Longint;                          {min position ...}
      tSlideColor:  TColor;                           {slider color}
      lSlidePos:    Longint;                          {slider position}
      bUnits:       Byte;                             {count of units to be drawn}
      fIsLoaded:    Boolean;                          {only for testing...}
      bWUnit:       Byte;                             {internal: width of slider}
      bHUnit:       Byte;                             {internal: height of slider}
      fShowUnits:   Boolean;                          {draw the unit lines}
      fAutoSlider:  Boolean;                          {automatic slider dimensions, based on heigth en width of panel}
      FOnChange:    TOnChange;                        {Event}
      FOnChanging:  TOnChanging;                      {Event}
      fSlidePressed:Boolean;                          {internal: slider has been activated}
      fShowText:    Boolean;                          {show the units text}
      fPanelWidth:  Byte;                             {Width of the Panel border}
      fPanelstate:  TPanelState;                      {5 pre-defined paneltypes}
      function      GetSlideX: Word;                  {returns slider position}
      procedure     DrawUnits(Canvas: TCanvas);       {draws the unit lines}
      procedure     DrawSliders(Canvas: TCanvas);     {draws the sliders}
      procedure     CreateProcedure; dynamic;         {procedure for initialization}
      function      GetPosition(X: Integer): Longint; {internal: returns position (from pixels to units)}
    protected
      procedure     SetMinValue(lValue: Longint);
      procedure     SetMaxValue(lValue: Longint);
      procedure     SetSlideColor(tValue: TColor);
      procedure     SetSlidePos(lValue: Longint);
      procedure     SetUnits(bValue: Byte);
      procedure     SetWUnit(bValue: Byte);
      procedure     SetHUnit(bValue: Byte);
      procedure     SetShowUnits(fValue: Boolean);
      procedure     SetAutoSlider(fValue: Boolean);
      procedure     SetShowText(fValue: Boolean);
      procedure     SetPanelWidth(fValue: Byte);
      Procedure     SetPanelState(fValue : TPanelState);

      procedure     MouseMove(Shift: TShiftState; X, Y: Integer); override;
      procedure     MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
      procedure     MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;

      procedure     Paint; override;
      procedure     SetSlider; dynamic;
    public
      constructor   Create(AOwner: TComponent); override;
      procedure     ForcePaint;
      destructor    Destroy; override;
    published
      Property      Align;
      property      OnChanging: TOnChanging read FOnChanging write FOnChanging;
      property      OnChange: TOnChange read FOnChange write FOnChange;
      property      ShowText: Boolean read fShowText write SetShowText default True;
      property      PanelState: TPanelstate read fPanelstate write SetPanelState default None;
      property      Panelwidth: Byte read fPanelWidth write SetPanelWidth default 1;
      property      Height default 32;
      property      Width  default 250;
      property      Top    default 10;
      property      Left   default 10;
      property      Hint;
      property      Font;
      property      Enabled;
      property      Visible;
      property      ShowUnits: Boolean read fShowUnits write SetShowUnits default True;
      property      AutoSlider: Boolean read fAutoSlider write SetAutoSlider default False;
      property      SliderWidth: Byte read bWUnit write SetWUnit default 8;
      property      SliderHeight: Byte read bHUnit write SetHUnit default 15;
      property      Units: Byte read bUnits write SetUnits default 10;
      property      SliderColor: TColor read tslideColor write SetSlideColor default clSilver;
      property      SliderPos: Longint read lSlidePos write SetSlidePos default 50;
      property      MaxValue: Longint read lMaxValue write SetMaxValue default 100;
      property      MinValue: Longint read lMinValue write SetMinValue default 0;
  end;

procedure Register;

implementation

constructor TLSlider.Create;
begin
  inherited Create(AOwner);
  fIsLoaded:=True;
  CreateProcedure;
end;

procedure  TLSlider.ForcePaint;
begin
  Paint;
end;

procedure  TLSlider.SetPanelWidth;
begin
  fPanelWidth:=fValue;
  if fPanelWidth> 2 then fPanelWidth := 2;
  InValidate;
end;

procedure  TLSlider.SetPanelState;
begin
  fPanelState:=fValue;
  InValidate;
end;

function   TLSlider.GetPosition;
begin
  GetPosition:=(Abs(lMaxValue)+Abs(lMinValue)) * X div Width;
end;

procedure  TLSlider.SetSlider;
begin
  If lSlidePos<lMinValue Then
    lSlidePos:=lMinValue;
  If lSlidePos>lMaxValue Then
    lSlidePos:=lMaxValue;
end;

procedure  TLSlider.SetShowText;
begin
  fShowText:=fValue;
  InValidate;
end;

procedure  TLSlider.MouseMove;
var        TChanged : TSlider;
           fChanged : Boolean;
begin
  TChanged:=esNoChange;
  fChanged:=False;
  If fSlidePressed then begin
    lSlidePos:=GetPosition(X)+lMinValue;
    TChanged:=esSlider;
    If lSlidePos>lMaxValue then
      lSlidePos:=lMaxValue;
    If lSlidePos<lMinValue then
      lSlidePos:=lMinValue;
    fChanged:=True;
  end;
  If Assigned(FOnChanging) Then
    FOnChanging(Self, lSlidePos, TChanged);
  SetSlider;
  If fChanged then
    Paint;
end;

procedure  TLSlider.MouseUp;
var        lPos    : Longint;
           tChanged: TSlider;
begin
  If fSlidePressed then
    tChanged:=esSlider;
  If Assigned(FOnChange) Then
    FOnChange(Self, lPos, tChanged);
  SetSlider;
  fSLidePressed:=False;
end;

procedure  TLSlider.MouseDown;
begin
  If Not fSlidePressed Then
    begin
      If (Y<=bHUnit+bHUnit div 5) And (Y>=1) And (X>=GetSlideX) And (X<=GetSlideX+bWUnit) Then
          fSlidePressed:=True;
    end;
end;

procedure  TLSlider.CreateProcedure;
begin
  ControlStyle:=[csClickEvents, csCaptureMouse];
  Width:=250;
  Height:=32;
  lSlidePos:=50;
  lMinValue:=0;
  lMaxValue:=100;
  bWUnit:=8;
  bHUnit:=15;
  bUnits:=10;
  Font.Name := 'Arial';
  Font.Size := 8;
  fPanelWidth :=1;
  fShowUnits:=True;
  fAutoSlider:=True;
  fShowText:=True;
end;

destructor TLSlider.Destroy;
begin
  inherited Destroy;
end;

procedure TLSlider.SetShowUnits;
begin
  fShowUnits:=fValue;
  InValidate;
end;

procedure TLSlider.SetAutoSlider;
begin
  fAutoSlider:=fValue;
  if fAutoSlider then begin
    bWUnit:=Width div 10;
    bHUnit:=Height div 8;
    InValidate;
  end;
end;

procedure TLSlider.SetWUnit;
begin
  If bValue>Width div 2 then
    bWUnit:=Width div 2
  else
    bWUnit:=bValue;
  SetAutoSlider(False);
Invalidate
end;

procedure TLSlider.SetHUnit;
begin
  If bValue>Height div 2 then
    bHUnit:=Height div 2
  else
    bHUnit:=bValue;
  SetAutoSlider(False);
Invalidate;
end;

procedure TLSlider.SetUnits;
begin
  If bValue>0 then
    bUnits:=bValue
  else
    Application.MessageBox('Sorry - At least one unit is necessary! Use ShowUnits() '+
                           'to disable units','Error',MB_ICONSTOP+MB_OK);
  InValidate;
end;

procedure TLSlider.SetSLidePos;
begin
  If (lValue<=lMaxValue) and (lValue>=lMinValue) Then begin
    lSlidePos:=lValue;
    InValidate;
  End;
end;

function  TLSlider.GetSlideX;
begin
  If Abs(lMaxValue)+Abs(lMinValue)>0 then
    GetSlideX:=(Width-bWUnit-2) * (lSlidePos-lMinValue) div (Abs(lMaxValue)+Abs(lMinValue))+1
  else
    GetSlideX:=1;
end;

procedure TLSlider.DrawUnits;
var       bCounter: Byte;
          bStrich : Byte;
          pc      : PChar;
          uiLeft  : Word;
          uiTop   : Word;
          uiRight : Word;
          uiBottom: Word;
          uiTW    : Word;
          uiTH    : Word;
begin
  Canvas.Font:=Font;
  bStrich:=(Height div 5);
  With Canvas do begin
    for bCounter:=0 to bUnits do begin
      if fShowUnits then begin
        Pen.Color:=clBtnShadow;
        MoveTo(Word(bCounter)*(Width-bWUnit-4) div (bUnits)+bWUnit div 2,bStrich);
        LineTo(Word(bCounter)*(Width-bWUnit-4) div (bUnits)+bWUnit div 2,2*bStrich);
        Pen.Color:=clBtnHighLight;
        MoveTo(Word(bCounter)*(Width-bWUnit-4) div (bUnits)+bWUnit div 2+1,bStrich);
        LineTo(Word(bCounter)*(Width-bWUnit-4) div (bUnits)+bWUnit div 2+1,2*bStrich);
      end;
      If fShowText Then Begin
        GetMem(pc,30);
        try
          StrPCopy(pc,IntToStr(((Abs(lMinValue)+Abs(lMaxValue)) * (bCounter) div bUnits)+lMinValue));
          uiTW:=TextWidth(StrPas(pc));
          uiTH:=TextHeight(StrPas(pc));
          If ((Longint(bCounter)*(Width-bWUnit-4) div (bUnits)+bWUnit div 2)-TextWidth(StrPas(pc)) div 2)>0 then
            uiLeft:=+(Longint(bCounter)*(Width-bWUnit-4) div (bUnits)+bWUnit div 2)-TextWidth(StrPas(pc)) div 2
          else
            uiLeft:=0;
          uiTop:=2*bStrich+1;
          uiRight:=+uiLeft+TextWidth(StrPas(pc));
          uiBottom:=uiTop+TextHeight(StrPas(pc));
          DrawText(Handle,pc,-1,Rect(uiLeft,uiTop,uiRight,uiBottom),
                   DT_CENTER or DT_NOPREFIX or DT_SINGLELINE);
        finally
          FreeMem(pc,30);
        end;
      end;
    end;
  end;
end;

procedure TLSlider.DrawSliders;
var tFrame  : TRect;
begin
  With Canvas do
    begin
      Tframe := Rect(4,bHunit div 2,Width-6,(bHunit div 6)+3);
      Frame3D(Canvas,tFrame,clBtnHighLight,clBtnShadow,1);
      Pen.Color:=clBlack;
      Brush.Color:=tSlideColor;
      PolyGon([Point(GetSlideX-1,2),                                  {.     }
               Point(GetSlideX-1,bHUnit+1),                           {     }
               Point(GetSlideX{-1}+bWUnit div 2,bHUnit+bHUnit div 5+1), {  \   }
               Point(GetSlideX+1+bWUnit,bHUnit+1),                    {   /  }
               Point(GetSlidex+bWUnit+1,2),                           {     }
               Point(GetSlideX-1,2)]);                                {     _}

      Pen.Color:=clBtnHighLight;
      PolyLine([Point(GetSlideX+bWUnit,3),
                Point(GetSlideX,3),
                Point(GetSlideX,BHUnit),
                Point(GetSlideX+bWUnit div 2,bHUnit+bHUnit div 5)]);

      Pen.Color:=clBtnShadow;
      PolyLine([Point(GetSLideX+bWUnit div 2,bHUnit+bHUnit div 5),
                Point(GetSlideX+bWUnit,bHUnit),
                Point(GetSlidex+bWUnit,1)]);
  end;
end;

procedure TLSlider.Paint;
var       bCounter: Byte;
          tBuffer : TBitMap;
          tFrame  : TRect;
begin
  If fAutoSlider then begin
    bWUnit:=Width div 10;
    bHUnit:=Height div 5;
  end;
  If Height<32 Then
    Height:=32;
  If Width<20 Then
    Width:=20;
  If Not fIsLoaded Then
    CreateProcedure;
  tBuffer:=TBitmap.Create;
  try
    tBuffer.Width:=Width;
    tBuffer.Height:=Height;
    With tBuffer do begin
      Canvas.Brush.Color:=clBtnFace;
      Canvas.FillRect(Rect(0,0,Width,Height));
      DrawUnits(Canvas);
      DrawSliders(Canvas);
      tFrame:=Rect(0,0,Width,Height);
      if fPanelState = Single then
      begin
      end;
      if fPanelState = Lowered then
        Frame3D(Canvas,tFrame,clBtnShadow,clBtnHighLight,fPanelWidth);
      if fPanelState = Raised then
        Frame3D(Canvas,tFrame,clBtnHighLight,clBtnShadow,fPanelWidth);
      if fPanelState = Group then
        begin
          tFrame:=Rect(0,0,Width,Height);
          Frame3D(Canvas,tFrame,clBtnShadow,clBtnHighLight,1);
          tFrame:=Rect(1,1,Width-1,Height-1);
          Frame3D(Canvas,tFrame,clBtnHighLight,clBtnShadow,1);
        end;
      if fPanelState = InvGroup then
        begin
          tFrame:=Rect(0,0,Width,Height);
          Frame3D(Canvas,tFrame,clBtnHighLight,clBtnShadow,1);
          tFrame:=Rect(1,1,Width-1,Height-1);
          Frame3D(Canvas,tFrame,clBtnShadow,clBtnHighLight,1);
        end
      else
      end;
    Canvas.Draw(0,0,tBuffer);
  finally
    tBuffer.Destroy;
  end;
end;

procedure TLSlider.SetMinValue;
begin
  lMinValue:=lValue;
  If (lSlidePos<lValue) Then
    lSlidePos:=lValue;
  InValidate;
end;

procedure TLSlider.SetMaxValue;
begin
  lMaxValue:=lValue;
  If (lSlidePos>lValue) Then
    lSlidePos:=lValue;
  InValidate;
end;

procedure TLSlider.SetSlideColor;
begin
  tSlideColor:=tValue;
  InValidate;
end;

procedure Register;
begin
  RegisterComponents('Examples', [TLSlider]);
end;

begin
end.
